@@ -95,6 +95,7 @@ private:
     int SkipFirstPayload(void);
     void ClientReverseFirstRead(void);
     void PostNullEvent(void);
+    inline bool WriteBB(void);
     Timestamp connect_done;
     bool peerclose;
     bool isburst;
@@ -116,6 +117,9 @@ private:
     struct msghdr message;
     char ctrl[CMSG_SPACE(sizeof(struct timeval))];
     struct cmsghdr *cmsg;
+#if HAVE_DECL_MSG_CTRUNC
+    bool ctrunc_warn_enable;
+#endif
 #endif
 #if defined(HAVE_LINUX_FILTER_H) && defined(HAVE_AF_PACKET)
     struct ether_header *eth_hdr;
