@@ -50,21 +50,28 @@
 
 #include "headers.h"
 
+#define HAVE_TCP_INFLIGHT (HAVE_TCP_STATS && HAVE_STRUCT_TCP_INFO_TCPI_DSACK_DUPS && \
+			   HAVE_STRUCT_TCP_INFO_TCPI_LOST && HAVE_STRUCT_TCP_INFO_TCPI_SACKED \
+			   && HAVE_STRUCT_TCP_INFO_TCPI_UNACKED)
+
 #ifdef __cplusplus
 extern "C" {
 #endif
 
 struct iperf_tcpstats {
     bool isValid;
-    int rtt;
+    uint32_t rtt;
     double connecttime;
 #if HAVE_TCP_STATS
-    int cwnd;
-    int rttvar;
-    intmax_t retry;
-    intmax_t retry_prev;
-    intmax_t retry_tot;
-    int mss_negotiated;
+    intmax_t cwnd;
+    uint32_t cwnd_packets;
+    uint32_t rttvar;
+    uint32_t retry;
+    uint32_t retry_prev;
+    uint32_t retry_tot;
+    uint32_t mss_negotiated;
+    uint32_t packets_in_flight;
+    intmax_t bytes_in_flight;
 #endif
 };
 
