@@ -1,8 +1,9 @@
 #include "config.h"
 #define IPERF_VERSION VERSION
-#define IPERF_VERSION_DATE "14 March 2023"
-#define IPERF_VERSION_MAJORHEX 0x00020001
-#define IPERF_VERSION_MINORHEX 0x00090003
+#define IPERF_VERSION_DATE "10 April 2024"
+#define IPERF_VERSION_MAJORHEX 0x00020002
+#define IPERF_VERSION_MINORHEX 0x00000003
+#define IPERF_BRANCH ""
 
 /*
  * 	case 0:
